package javax.swing;

import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleContext;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.beans.PropertyVetoException;
import java.util.ArrayList;
import java.util.TreeSet;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Collection;
import javax.swing.plaf.DesktopPaneUI;
import java.awt.Component;
import java.awt.Container;
import java.util.List;
import javax.accessibility.Accessible;

public class JDesktopPane extends JLayeredPane implements Accessible
{
    private static final String uiClassID = "DesktopPaneUI";
    transient DesktopManager desktopManager;
    private transient JInternalFrame selectedFrame;
    public static final int LIVE_DRAG_MODE = 0;
    public static final int OUTLINE_DRAG_MODE = 1;
    private int dragMode;
    private boolean dragModeSet;
    private transient List<JInternalFrame> framesCache;
    private boolean componentOrderCheckingEnabled;
    private boolean componentOrderChanged;
    
    public JDesktopPane() {
        this.selectedFrame = null;
        this.dragMode = 0;
        this.dragModeSet = false;
        this.componentOrderCheckingEnabled = true;
        this.componentOrderChanged = false;
        this.setUIProperty("opaque", Boolean.TRUE);
        this.setFocusCycleRoot(true);
        this.setFocusTraversalPolicy(new LayoutFocusTraversalPolicy() {
            @Override
            public Component getDefaultComponent(Container container) {
                JInternalFrame[] allFrames = JDesktopPane.this.getAllFrames();
                Component defaultComponent = null;
                int length = allFrames.length;
                int i = 0;
                while (i < length) {
                    JInternalFrame internalFrame = allFrames[i];
                    defaultComponent = internalFrame.getFocusTraversalPolicy().getDefaultComponent(internalFrame);
                    if (defaultComponent != null) {
                        break;
                    }
                    ++i;
                }
                return defaultComponent;
            }
        });
        this.updateUI();
    }
    
    public DesktopPaneUI getUI() {
        return (DesktopPaneUI)this.ui;
    }
    
    public void setUI(DesktopPaneUI ui) {
        super.setUI(ui);
    }
    
    public void setDragMode(int dragMode) {
        this.firePropertyChange("dragMode", this.dragMode, this.dragMode = dragMode);
        this.dragModeSet = true;
    }
    
    public int getDragMode() {
        return this.dragMode;
    }
    
    public DesktopManager getDesktopManager() {
        return this.desktopManager;
    }
    
    public void setDesktopManager(DesktopManager desktopManager) {
        this.firePropertyChange("desktopManager", this.desktopManager, this.desktopManager = desktopManager);
    }
    
    @Override
    public void updateUI() {
        this.setUI((DesktopPaneUI)UIManager.getUI(this));
    }
    
    @Override
    public String getUIClassID() {
        return "DesktopPaneUI";
    }
    
    public JInternalFrame[] getAllFrames() {
        return (JInternalFrame[])getAllFrames(this).toArray(new JInternalFrame[0]);
    }
    
    private static Collection<JInternalFrame> getAllFrames(Container container) {
        LinkedHashSet<JInternalFrame> set = new LinkedHashSet();
        for (int componentCount = container.getComponentCount(), i = 0; i < componentCount; ++i) {
            Component component = container.getComponent(i);
            if ((component instanceof JInternalFrame)) {
                set.add((JInternalFrame)component);
            }
            else if ((component instanceof JInternalFrame.JDesktopIcon)) {
                JInternalFrame internalFrame = ((JInternalFrame.JDesktopIcon)component).getInternalFrame();
                if (internalFrame != null) {
                    set.add(internalFrame);
                }
            }
            else if ((component instanceof Container)) {
                set.addAll(getAllFrames((Container)component));
            }
        }
        return set;
    }
    
    public JInternalFrame getSelectedFrame() {
        return this.selectedFrame;
    }
    
    public void setSelectedFrame(JInternalFrame selectedFrame) {
        this.selectedFrame = selectedFrame;
    }
    
    public JInternalFrame[] getAllFramesInLayer(int n) {
        Collection<JInternalFrame> allFrames = getAllFrames(this);
        Iterator<Object> iterator = allFrames.iterator();
        while (iterator.hasNext()) {
            if (((JInternalFrame)iterator.next()).getLayer() == n) {
                continue;
            }
            iterator.remove();
        }
        return (JInternalFrame[])allFrames.toArray(new JInternalFrame[0]);
    }
    
    private List<JInternalFrame> getFrames() {
        TreeSet<JDesktopPane.ComponentPosition> set = new TreeSet();
        for (int i = 0; i < this.getComponentCount(); ++i) {
            Component component = this.getComponent(i);
            if ((component instanceof JInternalFrame)) {
                set.add(new JDesktopPane.ComponentPosition((JInternalFrame)component, this.getLayer(component), i));
            }
            else if ((component instanceof JInternalFrame.JDesktopIcon)) {
                JInternalFrame internalFrame = ((JInternalFrame.JDesktopIcon)component).getInternalFrame();
                set.add(new JDesktopPane.ComponentPosition((JInternalFrame)internalFrame, this.getLayer(internalFrame), i));
            }
        }
        ArrayList o = new ArrayList(set.size());
        Iterator<Object> iterator = set.iterator();
        while (iterator.hasNext()) {
            o.add(((JDesktopPane.ComponentPosition)iterator.next()).component);
        }
        return o;
    }
    
    private JInternalFrame getNextFrame(JInternalFrame internalFrame, boolean b) {
        this.verifyFramesCache();
        if (internalFrame == null) {
            return this.getTopInternalFrame();
        }
        int index = this.framesCache.indexOf(internalFrame);
        if (index == -1 || this.framesCache.size() == 1) {
            return null;
        }
        if (b) {
            if (++index == this.framesCache.size()) {
                index = 0;
            }
        }
        else if (--index == -1) {
            index = this.framesCache.size() - 1;
        }
        return (JInternalFrame)this.framesCache.get(index);
    }
    
    JInternalFrame getNextFrame(JInternalFrame internalFrame) {
        return this.getNextFrame(internalFrame, true);
    }
    
    private JInternalFrame getTopInternalFrame() {
        if (this.framesCache.size() == 0) {
            return null;
        }
        return (JInternalFrame)this.framesCache.get(0);
    }
    
    private void updateFramesCache() {
        this.framesCache = this.getFrames();
    }
    
    private void verifyFramesCache() {
        if (this.componentOrderChanged) {
            this.componentOrderChanged = false;
            this.updateFramesCache();
        }
    }
    
    @Override
    public void remove(Component comp) {
        super.remove(comp);
        this.updateFramesCache();
    }
    
    public JInternalFrame selectFrame(boolean b) {
        JInternalFrame selectedFrame = this.getSelectedFrame();
        JInternalFrame nextFrame = this.getNextFrame(selectedFrame, b);
        if (nextFrame == null) {
            return null;
        }
        this.setComponentOrderCheckingEnabled(false);
        if (b && selectedFrame != null) {
            selectedFrame.moveToBack();
        }
        try {
            nextFrame.setSelected(true);
        }
        catch (PropertyVetoException ex) {}
        this.setComponentOrderCheckingEnabled(true);
        return nextFrame;
    }
    
    void setComponentOrderCheckingEnabled(boolean componentOrderCheckingEnabled) {
        this.componentOrderCheckingEnabled = componentOrderCheckingEnabled;
    }
    
    @Override
    protected void addImpl(Component component, Object o, int n) {
        super.addImpl(component, o, n);
        if (this.componentOrderCheckingEnabled && (component instanceof JInternalFrame || component instanceof JInternalFrame.JDesktopIcon)) {
            this.componentOrderChanged = true;
        }
    }
    
    @Override
    public void remove(int n) {
        if (this.componentOrderCheckingEnabled) {
            Component component = this.getComponent(n);
            if (component instanceof JInternalFrame || component instanceof JInternalFrame.JDesktopIcon) {
                this.componentOrderChanged = true;
            }
        }
        super.remove(n);
    }
    
    @Override
    public void removeAll() {
        if (this.componentOrderCheckingEnabled) {
            int componentCount = this.getComponentCount();
            int i = 0;
            while (i < componentCount) {
                Component component = this.getComponent(i);
                if (component instanceof JInternalFrame || component instanceof JInternalFrame.JDesktopIcon) {
                    this.componentOrderChanged = true;
                    break;
                }
                ++i;
            }
        }
        super.removeAll();
    }
    
    @Override
    public void setComponentZOrder(Component comp, int index) {
        super.setComponentZOrder(comp, index);
        if (this.componentOrderCheckingEnabled && (comp instanceof JInternalFrame || comp instanceof JInternalFrame.JDesktopIcon)) {
            this.componentOrderChanged = true;
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if (this.getUIClassID().equals("DesktopPaneUI")) {
            int n = (byte)(JComponent.getWriteObjCounter(this) - 1);
            JComponent.setWriteObjCounter(this, (byte)n);
            if (n == 0 && this.ui != null) {
                this.ui.installUI(this);
            }
        }
    }
    
    @Override
    void setUIProperty(String s, Object o) {
        if (s == "dragMode") {
            if (!this.dragModeSet) {
                this.setDragMode((int)o);
                this.dragModeSet = false;
            }
        }
        else {
            super.setUIProperty(s, o);
        }
    }
    
    @Override
    protected String paramString() {
        return super.paramString() + ",desktopManager=" + ((this.desktopManager != null) ? this.desktopManager.toString() : "");
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new JDesktopPane.AccessibleJDesktopPane();
        }
        return this.accessibleContext;
    }
    
    private static class ComponentPosition implements Comparable<JDesktopPane.ComponentPosition>
    {
        private final JInternalFrame component;
        private final int layer;
        private final int zOrder;
        
        ComponentPosition(JInternalFrame component, int layer, int zOrder) {
            this.component = component;
            this.layer = layer;
            this.zOrder = zOrder;
        }
        
        @Override
        public int compareTo(JDesktopPane.ComponentPosition componentPosition) {
            int n = componentPosition.layer - this.layer;
            if (n == 0) {
                return this.zOrder - componentPosition.zOrder;
            }
            return n;
        }
    }
    
    protected class AccessibleJDesktopPane extends JComponent.AccessibleJComponent
    {
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.DESKTOP_PANE;
        }
    }
}
